//////////////////////////////////////////////////////////////////////
// adsl_cfg.cpp - Device Manager Property Page for S518 ADSL adapter.
//
// Author	:	David Rokhvarg
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "sdlacfg.h"

////////////////////////////////////////////////////////////////////////////////
//AFT cfg functions prototypes
static
int 
ADSL_OnNotify(	
	HWND		ParentHwnd,
	LPNMHDR		NmHdr,
	SETUPSTUFF	*Params
	);

static
void 
ADSL_Init(
	HKEY	hkey, 
	HWND	hdlg,
	GLOBALS	*cfg
	);

static int selected_ind=0;

static BOOL ADSL_check_and_save_cfg(HWND hdlg);


///////////////////////////////////////////////////////////////////////////////

const char* encapsulation_1483_ROUTED_IP_LLC = "1483 ROUTED_IP_LLC";
const char* encapsulation_1483_ROUTED_IP_VC = "1483 ROUTED_IP_VC";

const char* encapsulation_1483_BRIDGED_ETH_LLC = "1483 BRIDGED_ETH_LLC";
const char* encapsulation_1483_BRIDGED_ETH_VC = "1483 BRIDGED_ETH_VC";

static
const char* ADSL_Encapsulation_options[] = 
	{	encapsulation_1483_ROUTED_IP_LLC,
		encapsulation_1483_ROUTED_IP_VC,
		encapsulation_1483_BRIDGED_ETH_LLC,
		encapsulation_1483_BRIDGED_ETH_VC,
		NULL
	};

static
const look_up_t AdslStandardOptions[] = {
	{"T1_413",		WANOPT_ADSL_T1_413},
	{"G_LITE",		WANOPT_ADSL_G_LITE},
	{"G_DMT",		WANOPT_ADSL_G_DMT},
	{"ALCATEL_1_4", WANOPT_ADSL_ALCATEL_1_4},
	{"MULTIMODE",	WANOPT_ADSL_MULTIMODE},
	{"ADI",			WANOPT_ADSL_ADI},
	{"ALCATEL",		WANOPT_ADSL_ALCATEL},
	{"T1_413_AUTO", WANOPT_ADSL_T1_413_AUTO},
	{NULL,			0}
};

///////////////////////////////////////////////////////////////////////////////
DLGPROC_RETURN_TYPE WINAPI ADSL_PageDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
{							// PageDlgProc
	switch (msg)
	{						// process message
	case WM_NOTIFY:
		Debug(DBG_ADSL_PAGE, "WM_NOTIFY");

		//If there is a problem with the new configuration 
		//refuse to close the property page and return TRUE.
		//If all checks are OK allow the closing.
        if(ADSL_OnNotify(hdlg, (NMHDR *)lParam, stuff) == HANDLE_ERROR)
		{
			Debug(DBG_ADSL_PAGE, "Invalid Configuration, refusing to close!");
			SetWindowLong(hdlg, DWL_MSGRESULT, PSNRET_INVALID);
			return TRUE;
		}
		break;

	case WM_INITDIALOG:
		{
			if (okay == TRUE)
			{
				//initialize visual controls
				ADSL_Init(hkey, hdlg, &Globals_New_Cfg);
			}

			if (!okay)
			{					// no SampleInfo
				ShowWindow(GetDlgItem(hdlg, IDC_INFOLABEL), SW_HIDE);
				ShowWindow(GetDlgItem(hdlg, IDB_MOREINFO), SW_HIDE);
			}					// no SampleInfo
		}
		break;
		
   	case WM_DESTROY:
		Debug(DBG_ADSL_PAGE, "Samclass:WM_DESTROY");
		{
			LPSTR infofile = (LPSTR) GetWindowLongPtr(hdlg, DWL_USER);
			if (infofile){
				GlobalFree((HGLOBAL) infofile);
			}
		}
		break;

	case WM_SHOWWINDOW:
		//This message received each time the Dialog shown/hidden.
		//Update visual controls with latest information stored in "Globals_New_Cfg".
		Debug(DBG_ADSL_PAGE, "DBG_ADSL_PAGE:WM_SHOWWINDOW");
		set_dialog_wanpipe_name(hdlg);
		ADSL_Init(hkey, hdlg, &Globals_New_Cfg);
		break;

    case WM_COMMAND:
		switch (LOWORD(wParam))
		{					// process command notification
		case BUTTON_HELP:
			WinHelp(GetActiveWindow(), "WanpipePnp.hlp", HELP_CONTEXT,
				IDH_LINE_CFG);
			break;

		case BUTTON_START_MONITOR:
			start_monitor(selected_ind);
			break;
		}					// process command notification
		return TRUE;
	}						// process message

	return FALSE;
}							// PageDlgProc

///////////////////////////////////////////////////////////////////////////////

UINT CALLBACK ADSL_PageCallbackProc(HWND junk, UINT msg, LPPROPSHEETPAGE p)
{							// PageCallbackProc
	if (msg == PSPCB_RELEASE && p->lParam)
	{
		Debug(DBG_ADSL_PAGE, "PSPCB_RELEASE : ADSL_PageCallbackProc()");

		RegCloseKey(hkey);

		//compare the configuration structures, restart if different.
		if((released_by_OK_button == 1) &&
		   memcmp(&Globals_Old_Cfg, &Globals_New_Cfg, sizeof(Globals_New_Cfg)))
		{
			if(yes_no_question("Device Configuration was changed. Do you want the Device to be restarted now?")
					== IDYES){
				restart_device();
			}else{
				MessageBox(GetActiveWindow(), (LPCTSTR)"Device was NOT restarted.",
					"Note for the user", MB_OK | MB_ICONINFORMATION | MB_SYSTEMMODAL );
			}
		}
		delete (SETUPSTUFF*) p->lParam;
		return 0;//return code is ignored
	}
	return TRUE;				// matters only for PSPCB_CREATE
}							// PageCallbackProc

///////////////////////////////////////////////////////////////////////////////

static int ADSL_OnNotify(	HWND    ParentHwnd,
							LPNMHDR NmHdr,
							SETUPSTUFF * Params	)
{
    SP_DEVINSTALL_PARAMS spDevInstall = {0};
    TCHAR                friendlyName[LINE_LEN] ={0};
	BOOL	configuration_check_passed = FALSE;
    
    switch (NmHdr->code) 
	{
    case PSN_APPLY:
        //
        // Sent when the user clicks on Apply OR OK !!
        //
		configuration_check_passed = ADSL_check_and_save_cfg(ParentHwnd);

		Debug(DBG_ADSL_PAGE, "AFT_OnNotify((): PSN_APPLY");

		released_by_OK_button = 1;

		if(configuration_check_passed == TRUE){
			return HANDLE_OK;
		}else{
			return HANDLE_ERROR;
		}
		break;

	case PSN_QUERYCANCEL:
		Debug(DBG_ADSL_PAGE, "AFT_OnNotify(() : PSN_QUERYCANCEL");
		released_by_OK_button = 0;
    default:
		Debug(DBG_ADSL_PAGE, "AFT_OnNotify(() : default case:");
        return HANDLE_DEFAULT;
    }
    return HANDLE_DEFAULT;   
}

///////////////////////////////////////////////////////////////////////////////

static BOOL ADSL_check_and_save_cfg(HWND hdlg)
{	
	unsigned int	j;
	char			local_tmp_buff[TMP_BUFFER_LEN];

	Debug(DBG_ADSL_PAGE, "ADSL_check_and_save_cfg()");
	
	Debug(DBG_ADSL_PAGE, "*********  Writing ADSL Interface Configuration  ********\n");

	///////////////////////////////////////////////////////////////////
	//get vpi
	GetDlgItemText(hdlg, IDC_EDIT_VPI, local_tmp_buff, TMP_BUFFER_LEN);
	SetRegistryStringValue(hkey, "AdslVpi", local_tmp_buff);
	Globals_New_Cfg.adsl_cfg.Vpi = (unsigned short)atoi(local_tmp_buff);

	//get vci
	GetDlgItemText(hdlg, IDC_EDIT_VCI, local_tmp_buff, TMP_BUFFER_LEN);
	SetRegistryStringValue(hkey, "AdslVci", local_tmp_buff);
	Globals_New_Cfg.adsl_cfg.Vci = (unsigned short)atoi(local_tmp_buff);

	//get rx cell count
	GetDlgItemText(hdlg, IDC_EDIT_RX_CELL_CNT, local_tmp_buff, TMP_BUFFER_LEN);
	SetRegistryStringValue(hkey, "AdslRxCellCount", local_tmp_buff);
	Globals_New_Cfg.adsl_cfg.RxCellCount = (unsigned short)atoi(local_tmp_buff);

	j = atoi(local_tmp_buff);
	if(j < 1 || j > 40){

		_snprintf(local_tmp_buff, TMP_BUFFER_LEN,
			"Invalid value in \"RxCellCount\". Minimum is %d, Maximum is %d.",
			1, 40);

		MessageBox(GetActiveWindow(), local_tmp_buff, "Configuration Error",															//szTitleString,
			MB_OK|MB_ICONWARNING|MB_SYSTEMMODAL);
		return FALSE;
	}

	//???????????????????
	//get AdslTxPowerAtten
	GetDlgItemText(hdlg, IDC_EDIT_ADSL_TX_POWER_ATTENUATION, local_tmp_buff, TMP_BUFFER_LEN);
	SetRegistryStringValue(hkey, "AdslTxPowerAtten", local_tmp_buff);
	Globals_New_Cfg.adsl_cfg.TxPowerAtten = atoi(local_tmp_buff);

	//get AdslCodingGain
	GetDlgItemText(hdlg, IDC_EDIT_ADSL_CODING_GAIN, local_tmp_buff, TMP_BUFFER_LEN);
	SetRegistryStringValue(hkey, "AdslCodingGain", local_tmp_buff);
	Globals_New_Cfg.adsl_cfg.CodingGain = atoi(local_tmp_buff);
	//???????????????????

	///////////////////////////////////////////////////////////////////
	//get Encapsulation type
	j = SendDlgItemMessage(hdlg, IDC_COMBO_ADSL_RFC, CB_GETCURSEL, 0, 0);
	switch(j)
	{
	case 0:
		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", RFC_MODE_ROUTED_IP_LLC);
		Globals_New_Cfg.adsl_cfg.EncapMode = RFC_MODE_ROUTED_IP_LLC;
		break;

	case 1:
		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", RFC_MODE_ROUTED_IP_VC);
		Globals_New_Cfg.adsl_cfg.EncapMode = RFC_MODE_ROUTED_IP_VC;
		break;

	case 2:
		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", RFC_MODE_BRIDGED_ETH_LLC);
		Globals_New_Cfg.adsl_cfg.EncapMode = RFC_MODE_BRIDGED_ETH_LLC;
		break;

	case 3:
		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", RFC_MODE_BRIDGED_ETH_VC);
		Globals_New_Cfg.adsl_cfg.EncapMode = RFC_MODE_BRIDGED_ETH_VC;
		break;

	default:
		snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", RFC_MODE_ROUTED_IP_LLC);
		Globals_New_Cfg.adsl_cfg.EncapMode = RFC_MODE_ROUTED_IP_LLC;
		Debug(DBG_ADSL_PAGE, "Unsupported ADSL RFC!!\n");
	}
	SetRegistryStringValue(hkey, "AdslEncapMode", local_tmp_buff);

	///////////////////////////////////////////////////////////////////
	//get ADSL Standard
	j = SendDlgItemMessage(hdlg, IDC_COMBO_ADSL_STANDARD, CB_GETCURSEL, 0, 0);
	Globals_New_Cfg.adsl_cfg.Standard = AdslStandardOptions[j].value;
	SetRegistryStringValue(hkey, "AdslStandard", Globals_New_Cfg.adsl_cfg.Standard);

	Debug(DBG_ADSL_PAGE, "*********  Finished writing ADSL Interface Configuration  ********\n");
	///////////////////////////////////////////////////////////////////

	return TRUE;
}
///////////////////////////////////////////////////////////////////////////////

static void ADSL_Init(HKEY	hkey, HWND hdlg, GLOBALS* cfg)
{
	int i, current_selection_index;
	static char local_tmp_buff[TMP_BUFFER_LEN];

	///////////////////////////////////////////////////////////////////////
	//set the vpi value
	snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", Globals_New_Cfg.adsl_cfg.Vpi);
	SetDlgItemText(hdlg, IDC_EDIT_VPI, local_tmp_buff);

	//set the vci value
	snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", Globals_New_Cfg.adsl_cfg.Vci);
	SetDlgItemText(hdlg, IDC_EDIT_VCI, local_tmp_buff);

	//set the rx cell count value
	snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", Globals_New_Cfg.adsl_cfg.RxCellCount);
	SetDlgItemText(hdlg, IDC_EDIT_RX_CELL_CNT, local_tmp_buff);

	//???????????????????
	//set AdslTxPowerAtten
	snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", Globals_New_Cfg.adsl_cfg.TxPowerAtten);
	SetDlgItemText(hdlg, IDC_EDIT_ADSL_TX_POWER_ATTENUATION, local_tmp_buff);

	//set AdslCodingGain
	snprintf(local_tmp_buff, TMP_BUFFER_LEN, "%d", Globals_New_Cfg.adsl_cfg.CodingGain);
	SetDlgItemText(hdlg, IDC_EDIT_ADSL_CODING_GAIN, local_tmp_buff);
	//???????????????????

	///////////////////////////////////////////////////////////////////////
	//add strings to ADSL_Encapsulation_options combobox
	SendDlgItemMessage(hdlg, IDC_COMBO_ADSL_RFC, CB_RESETCONTENT, 0, 0);
	i = 0;
	while(ADSL_Encapsulation_options[i] != NULL){
		SendDlgItemMessage(hdlg, IDC_COMBO_ADSL_RFC, CB_ADDSTRING, 0,
			(LPARAM)ADSL_Encapsulation_options[i]);
		i++;
	}

	//make the current one selected
	switch(Globals_New_Cfg.adsl_cfg.EncapMode)
	{
    case RFC_MODE_ROUTED_IP_LLC://      = 2,
		i = SendDlgItemMessage(	hdlg, IDC_COMBO_ADSL_RFC, CB_FINDSTRING, -1,
					(LPARAM)encapsulation_1483_ROUTED_IP_LLC);
		break;

    case RFC_MODE_ROUTED_IP_VC://       = 3,
		i = SendDlgItemMessage(	hdlg, IDC_COMBO_ADSL_RFC, CB_FINDSTRING, -1,
					(LPARAM)encapsulation_1483_ROUTED_IP_VC);
		break;

    case RFC_MODE_BRIDGED_ETH_LLC://
		i = SendDlgItemMessage(	hdlg, IDC_COMBO_ADSL_RFC, CB_FINDSTRING, -1,
					(LPARAM)encapsulation_1483_BRIDGED_ETH_LLC);
		break;

    case RFC_MODE_BRIDGED_ETH_VC://
		i = SendDlgItemMessage(	hdlg, IDC_COMBO_ADSL_RFC, CB_FINDSTRING, -1,
					(LPARAM)encapsulation_1483_BRIDGED_ETH_VC);
		break;

	}
	SendDlgItemMessage(hdlg, IDC_COMBO_ADSL_RFC, CB_SETCURSEL, (i != CB_ERR ? i : 0), 0);

	///////////////////////////////////////////////////////////////////////
	//add strings to IDC_COMBO_ADSL_STANDARD combobox
	SendDlgItemMessage(hdlg, IDC_COMBO_ADSL_STANDARD, CB_RESETCONTENT, 0, 0);
	i = current_selection_index = 0;
	while(AdslStandardOptions[i].name != NULL){
		SendDlgItemMessage(hdlg, IDC_COMBO_ADSL_STANDARD, CB_ADDSTRING, 0, (LPARAM)AdslStandardOptions[i].name);
		if(AdslStandardOptions[i].value == Globals_New_Cfg.adsl_cfg.Standard){
			current_selection_index = i;
		}
		i++;
	}
	//make the current one selected
	SendDlgItemMessage(hdlg, IDC_COMBO_ADSL_STANDARD, CB_SETCURSEL, current_selection_index, 0);

}

//////////////////////////////////////////////////////////////////////////////

